@@ -6,9 +6,7 @@
 
 #define G_LOG_DOMAIN "cm-net"
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+#include "cm-config.h"
 
 #define GCRYPT_NO_DEPRECATED
 #include <gcrypt.h>
